@@ -4,7 +4,7 @@
 # Created: 4/4/20
 
 
-__version__ = '0.3.8'
+__version__ = '0.4.0'
 __description__ = 'mtdata is a tool to download datasets for machine translation'
 __author__ = 'Thamme Gowda'
 
@@ -21,10 +21,12 @@ log.basicConfig(level=log.INFO, datefmt='%Y-%m-%d %H:%M:%S', format=_log_format)
 cache_dir = Path(os.environ.get('MTDATA', '~/.mtdata')).expanduser()
 recipes_dir = Path(os.getenv('MTDATA_RECIPES', '.')).resolve()
 cached_index_file = cache_dir / f'mtdata.index.{__version__}.pkl'
-
-FILE_LOCK_TIMEOUT = 2 * 60 * 60  # 2 hours
+resource_dir:Path = Path(__file__).parent / 'resource'
 pbar_man = enlighten.get_manager()
 
-
 class MTDataException(Exception):
     pass
+
+class Defaults:
+    FILE_LOCK_TIMEOUT = 2 * 60 * 60  # 2 hours
+    PBAR_REFRESH_INTERVAL = 1    # seconds
\ No newline at end of file
