@@ -8,7 +8,7 @@ import fnmatch
 from dataclasses import dataclass
 from pathlib import Path
 from mtdata.index import Entry
-from mtdata import log, __version__, pbar_man, MTDataException, FILE_LOCK_TIMEOUT
+from mtdata import log, __version__, pbar_man, MTDataException, Defaults
 from mtdata.utils import ZipPath, TarPath, format_byte_size
 from mtdata.parser import Parser
 from typing import List, Union, Dict, Any
@@ -32,22 +32,27 @@ class Cache:
     def __post_init__(self):
         if isinstance(self.root, str):
             self.root = Path(self.root)
-        log.info(f"Local cache is at {self.root}")
+        log.debug(f"Local cache is at {self.root}")
 
     def get_entry(self, entry: Entry, fix_missing=True) -> Union[Path, List[Path]]:
         if entry.in_ext == OPUS_XCES:
             return self.opus_xces_format(entry=entry, fix_missing=fix_missing)
-
-        if isinstance(entry.url, (list, tuple)):
-            assert isinstance(entry.url[0], str)
-            local = [self.get_local_path(url, fix_missing=fix_missing) for url in entry.url]
-        else:
-            assert isinstance(entry.url, str)
-            local = self.get_local_path(entry.url, filename=entry.filename, fix_missing=fix_missing)
-            if zipfile.is_zipfile(local) or tarfile.is_tarfile(local):
-                # look inside the archives and get the desired files
-                local = self.get_local_in_paths(path=local, entry=entry)
-        return local
+        local = None
+        try:
+            if isinstance(entry.url, (list, tuple)):
+                assert isinstance(entry.url[0], str)
+                local = [self.get_local_path(url, fix_missing=fix_missing, entry=entry) for url in entry.url]
+            else:
+                assert isinstance(entry.url, str)
+                local = self.get_local_path(entry.url, filename=entry.filename, fix_missing=fix_missing, entry=entry)
+                if zipfile.is_zipfile(local) or tarfile.is_tarfile(local):
+                    # look inside the archives and get the desired files
+                    local = self.get_local_in_paths(path=local, entry=entry)
+            return local
+        except:
+            if local:
+                log.warning(f'Error while accessing {entry.did} --> {local}')
+            raise
 
     def get_content_length(self, entry: Entry) -> Dict[str, Any]:
         urls = []
@@ -126,14 +131,18 @@ class Cache:
     def get_flag_file(self, file: Path):
         return file.with_name(file.name + '._valid')
 
-    def get_local_path(self, url, filename=None, fix_missing=True):
+    def get_local_path(self, url, filename=None, fix_missing=True, entry=None):
         hostname = urlparse(url).hostname or 'nohost'
         filename = filename or url.split('/')[-1]
         assert hostname and filename
         mdf5_sum = md5(url.encode('utf-8')).hexdigest()
         local = self.root / hostname / mdf5_sum[:4] / mdf5_sum[4:] / filename
         if fix_missing:
-            self.download(url, local)
+            try:
+                self.download(url, local, entry=entry)
+            except:
+                log.error(f'Error downloading {entry and entry.did}\nURL: {url}\nPath:{local}')
+                raise
         return local
 
     @classmethod
@@ -149,9 +158,9 @@ class Cache:
     def opus_xces_format(self, entry, fix_missing=True) -> List[Path]:
         assert entry.in_ext == OPUS_XCES
         l1_url, l2_url = entry.in_paths
-        align_file = self.get_local_path(entry.url, fix_missing=fix_missing)
-        l1_path = self.get_local_path(l1_url, fix_missing=fix_missing)
-        l2_path = self.get_local_path(l2_url, fix_missing=fix_missing)
+        align_file = self.get_local_path(entry.url, fix_missing=fix_missing, entry=entry)
+        l1_path = self.get_local_path(l1_url, fix_missing=fix_missing, entry=entry)
+        l2_path = self.get_local_path(l2_url, fix_missing=fix_missing, entry=entry)
         return [align_file, l1_path, l2_path]
 
     def get_local_in_paths(self, path: Path, entry: Entry,):
@@ -165,29 +174,34 @@ class Cache:
         else:
             raise MTDataException(f'Unable to read {entry.did}; the file is neither zip nor tar')
 
-    def download(self, url: str, save_at: Path, timeout=(5, 10)):
+    def download(self, url: str, save_at: Path, timeout=(5, 10), entry=None):
+        
         valid_flag = self.get_flag_file(save_at)
         lock_file = valid_flag.with_suffix("._lock")
         if valid_flag.exists() and save_at.exists():
             return save_at
         save_at.parent.mkdir(parents=True, exist_ok=True)
-
-        log.info(f"Acquiring lock on {lock_file}")
-        with portalocker.Lock(lock_file, 'w', timeout=FILE_LOCK_TIMEOUT) as fh:
+        log.info(f"Download: {url} → {save_at}")
+        log.debug(f"Acquiring lock on {lock_file}")
+        with portalocker.Lock(lock_file, 'w', timeout=Defaults.FILE_LOCK_TIMEOUT) as fh:
             # check if downloaded by  other parallel process
             if valid_flag.exists() and save_at.exists():
                 return save_at
-            log.info(f"GET {url} → {save_at}")
+            log.debug(f"GET {url} → {save_at}")
             resp = requests.get(url=url, allow_redirects=True, headers=headers, stream=True, timeout=timeout)
             assert resp.status_code == 200, resp.status_code
             buf_size = 2 ** 14
             tot_bytes = int(resp.headers.get('Content-Length', '0'))
             n_buffers = math.ceil(tot_bytes / buf_size) or None
-            desc = url
-            if len(desc) > 60:
-                desc = desc[:30] + '...' + desc[-28:]
+            parts = url.split('/')  
+            desc = [entry and f'{entry.did} |' or '',
+                    tot_bytes and (format_byte_size(tot_bytes) + "|") or "",
+                    parts[2][:24], '...', parts[-1][-24:], # host ... filename
+                    ]
+            desc = ''.join(desc) 
             with pbar_man.counter(color='green', total=tot_bytes//2**10, unit='KiB', leave=False, position=2,
-                                  desc=f"{desc}") as pbar, open(save_at, 'wb', buffering=2**24) as out:
+                                  min_delta=Defaults.PBAR_REFRESH_INTERVAL, desc=f"{desc}"
+                                  ) as pbar, open(save_at, 'wb', buffering=2**24) as out:
                 for chunk in resp.iter_content(chunk_size=buf_size):
                     out.write(chunk)
                     pbar.update(incr=buf_size//2**10)
