@@ -7,11 +7,11 @@ import json
 from itertools import zip_longest
 from multiprocessing import Pool
 from pathlib import Path
-from typing import Optional, List, Tuple
+from typing import Optional, List, Tuple, Dict, Union
 
 import portalocker
 
-from mtdata import log, pbar_man, cache_dir as CACHE_DIR, FILE_LOCK_TIMEOUT
+from mtdata import log, pbar_man, cache_dir as CACHE_DIR, Defaults
 from mtdata.cache import Cache
 from mtdata.entry import Entry, DatasetId, LangPair
 from mtdata.index import INDEX
@@ -20,6 +20,7 @@ from mtdata.parser import Parser
 from mtdata.utils import IO
 
 DEF_COMPRESS = 'gz'
+DATA_FIELDS = ('train', 'dev', 'test', 'mono_train', 'mono_dev', 'mono_test')
 
 
 class Dataset:
@@ -27,7 +28,6 @@ class Dataset:
     def __init__(self, dir: Path, langs: LangPair, cache_dir: Path, drop_train_noise=True, n_jobs=1,
                  drop_test_noise=False, drop_dupes=False, drop_tests=False, compress=False, fail_on_error=False):
         self.dir = dir
-
         assert len(langs) == 2, f'Only parallel datasets are supported for now and expected two langs; {langs}'
         assert isinstance(langs[0], BCP47Tag)
         assert isinstance(langs[1], BCP47Tag)
@@ -36,6 +36,8 @@ class Dataset:
 
         self.train_parts_dir = dir / 'train-parts'  # will be merged later
         self.tests_dir = dir / 'tests'  # wont be merged
+        self.mono_train_parts_dir = dir / 'mono-train-parts'  # wil be merged
+        self.mono_tests_dir = dir / 'mono-tests'        # wont be merged
         self.train_parts_dir.mkdir(parents=True, exist_ok=True)
         self.tests_dir.mkdir(parents=True, exist_ok=True)
         self.drop_train_noise = drop_train_noise
@@ -63,19 +65,21 @@ class Dataset:
         return entries
 
     @classmethod
-    def prepare(cls, langs, out_dir: Path, train_dids: Optional[List[DatasetId]] = None,
-                test_dids: Optional[List[DatasetId]] = None, dev_dids: Optional[List[DatasetId]] = None,
+    def prepare(cls, langs, out_dir: Path, dataset_ids=Dict[str, List[DatasetId]],
                 cache_dir: Path = CACHE_DIR, merge_train=False, drop_noise: Tuple[bool, bool] = (True, False),
                 compress=False, drop_dupes=False, drop_tests=False, fail_on_error=False, n_jobs=1):
         drop_train_noise, drop_test_noise = drop_noise
         assert langs, 'langs required'
-        assert train_dids or test_dids, 'Either train_names or test_names should be given'
+        assert dataset_ids
+        assert isinstance(dataset_ids, dict)
+        err_keys = dataset_ids.keys() - set(DATA_FIELDS)
+        assert not err_keys, f'{err_keys} are unknown. Supported fields: {DATA_FIELDS}'
         # First, resolve and check if they exist before going to process them.
         # Fail early for typos in name
-        all_dids = (train_dids or []) + (dev_dids or []) + (test_dids or [])
+        all_dids = [id for ids in dataset_ids.values() for id in (ids or [])]
         all_entries = cls.resolve_entries(all_dids)
         for ent in all_entries:
-            if not BCP47Tag.check_compat_swap(pair1=langs, pair2=ent.did.langs)[0]:
+            if not ent.is_compatible(langs):
                 raise Exception(f'Given languages: {langs} and dataset: {ent.did} are not compatible')
 
         dataset = cls(dir=out_dir, langs=langs, cache_dir=cache_dir, drop_train_noise=drop_train_noise,
@@ -83,14 +87,14 @@ class Dataset:
                       fail_on_error=fail_on_error, n_jobs=n_jobs)
 
         dev_entries, test_entries = [], []
-        if test_dids:  # tests are smaller so quicker; no merging needed
-            test_entries = cls.resolve_entries(test_dids)
+        if dataset_ids.get('test'):  # tests are smaller so quicker; no merging needed
+            test_entries = cls.resolve_entries(dataset_ids['test'])
             dataset.add_test_entries(test_entries)
-        if dev_dids:
-            dev_entries = cls.resolve_entries(dev_dids)
+        if dataset_ids.get('dev'):
+            dev_entries = cls.resolve_entries(dataset_ids['dev'])
             dataset.add_dev_entries(dev_entries)
-        if train_dids:  # this might take some time
-            train_entries = cls.resolve_entries(train_dids)
+        if dataset_ids.get('train'):  # this might take some time
+            train_entries = cls.resolve_entries(dataset_ids['train'])
             drop_hashes = None
             if drop_tests:
                 pair_files = []
@@ -103,6 +107,14 @@ class Dataset:
                 drop_hashes = test_pair_hash | test_seg_hash  # set union
             dataset.add_train_entries(train_entries, merge_train=merge_train, compress=compress,
                                       drop_hashes=drop_hashes)
+        for key, dirpath in [('mono_train', dataset.mono_train_parts_dir), 
+                             ('mono_dev', dataset.mono_tests_dir), 
+                             ('mono_test', dataset.mono_tests_dir)]:
+            if dataset_ids.get(key):
+                dirpath.mkdir(exist_ok=True)
+                entries = cls.resolve_entries(dataset_ids[key])
+                for entry in entries:
+                    dataset.add_mono_entry(dirpath, entry, compress=compress) 
 
         # citations
         refs_file = out_dir / 'references.bib'
@@ -110,7 +122,15 @@ class Dataset:
             refs_file.rename(refs_file.with_suffix('.bib.bak'))
         with refs_file.open('w', encoding='utf-8', errors='ignore') as fh:
             for ent in all_entries:
-                cite = ent.cite or 'UNKNOWN'
+                cite = ent.cite
+                if cite:
+                    try:
+                        if isinstance(cite, str):
+                            cite = [cite]
+                        cite = '\n'.join(INDEX.ref_db.get_bibtex(key) for key in cite)
+                    except:
+                        log.exception(f'Error reading citation for {ent.did}: {cite}', exc_info=True)
+                cite = cite or '%% UNKNOWN'
                 fh.write(f"%% {ent.did}\n{cite}\n\n")
         log.info(f"Created references at {refs_file}")
         return dataset
@@ -135,7 +155,7 @@ class Dataset:
         # paired_files = self.find_bitext_pairs(self.train_parts_dir, lang1, lang2)
         paired_files = {}
         for ent in entries:
-            e1, e2 = self.get_paths(self.train_parts_dir, ent)
+            e1, e2 = self.get_paths(self.train_parts_dir, ent, compress=compress)
             _, swapped = BCP47Tag.check_compat_swap(self.langs, ent.did.langs, fail_on_incompat=True)
             if swapped:
                 e1, e2 = e2, e1
@@ -156,8 +176,8 @@ class Dataset:
         train_hashes = set()
 
         with IO.writer(of1) as w1, IO.writer(of2) as w2, IO.writer(of3) as w3:
-            with pbar_man.counter(color='green', total=len(paired_files), unit='it', desc="Merging",
-                                  autorefresh=False) as pbar:
+            with pbar_man.counter(color='green', total=len(paired_files), unit='it', desc="Merging", leave=False,
+                                  min_delta=Defaults.PBAR_REFRESH_INTERVAL, autorefresh=True) as pbar:
                 for name, (if1, if2) in paired_files.items():
                     for seg1, seg2 in self.read_parallel(if1, if2):
                         counts['total'][name] += 1
@@ -186,6 +206,38 @@ class Dataset:
         log.info('Train stats:\n' + stats_msg)
         IO.write_lines(self.dir / 'train.stats.json', stats_msg)
         return counts
+    
+    def add_mono_entry(self, dirpath, entry: Entry, compress=False):
+        flag_file = dirpath / f'.valid.{entry.did}'
+        assert len(entry.did.langs) == 1, f'Monolingual entry expected, given {entry.did}'
+        if flag_file.exists():
+            log.info(f"{flag_file} exits. Skipping")
+            return -1, -1
+        cache_path = self.cache.get_entry(entry)
+        parser = Parser(cache_path, ext=entry.in_ext or None, ent=entry)
+        out_path = self.get_paths(dirpath, entry, compress=compress)
+        log.info("Writing %s to %s", entry.did, out_path)
+        io_args = dict(encoding='utf-8', errors='ignore')
+        with IO.writer(out_path, **io_args) as out:
+            count, skips = 0, 0
+            for sentence in parser.read_segs():
+                if isinstance(sentence, (list, tuple)) and len(sentence) == 1:
+                    sentence = sentence[0]   # flatten list
+                assert isinstance(sentence, str), f'str sentence expected. found: {type(sentence)}; entry: {entry.did}'
+                sentence = sentence and sentence.strip()
+                if not sentence:
+                    skips += 1
+                    continue
+                sentence = sentence.replace('\n', ' ').replace('\t', ' ').replace('\r', ' ')
+                out.write(f'{sentence}\n')
+                count += 1
+            msg = f'Looks like an error. {count} segs are valid {skips} are invalid: {entry}'
+            assert count > 0, msg
+            if skips > count:
+                log.warning(msg)
+                log.info(f"{entry}: Skips : {skips:,}/{count:,} => {100 * skips / count:.4f}%")
+        flag_file.touch()
+        return count, skips
 
     @classmethod
     def find_bitext_pairs(cls, dir_path: Path, lang1: BCP47Tag, lang2: BCP47Tag):
@@ -274,7 +326,8 @@ class Dataset:
         of1, of2 = out_paths
         of1.parent.mkdir(exist_ok=True)
         of2.parent.mkdir(exist_ok=True)
-        with pbar_man.counter(color='green', total=len(in_paths), unit='it', desc="Merging") as pbar, \
+        with pbar_man.counter(color='green', total=len(in_paths), unit='it', desc="Merging", leave=False,
+                              min_delta=Defaults.PBAR_REFRESH_INTERVAL, autorefresh=True) as pbar, \
                 IO.writer(of1) as w1, IO.writer(of2) as w2:
             for if1, if2 in in_paths:
                 assert if1.exists()
@@ -297,7 +350,7 @@ class Dataset:
             if fail_on_error:
                 raise e
             msg = str(e).replace('\n', '\t')
-            with portalocker.Lock(self.errors_file, 'a', timeout=FILE_LOCK_TIMEOUT) as fh:
+            with portalocker.Lock(self.errors_file, 'a', timeout=Defaults.FILE_LOCK_TIMEOUT) as fh:
                 # self.errors_file.open('a').write(f"{ent.did}\t{msg}\n")
                 fh.write(f"{ent.did}\t{msg}\n")
 
@@ -311,13 +364,13 @@ class Dataset:
                       fail_on_error=fail_on_error) for ent in entries]
         pool = Pool(self.n_jobs)
         with pbar_man.counter(color='blue', leave=False, total=len(entries), unit='it', desc=desc,
-                              autorefresh=True, position=3) as pbar:
+                              autorefresh=True, min_delta=Defaults.PBAR_REFRESH_INTERVAL, position=3) as pbar:
             for _ in pool.imap_unordered(self.add_part_thread, tasks):
                 pbar.update(force=True)
 
     def add_parts_sequential(self, dir_path, entries, drop_noise=False, compress=False, desc=None, fail_on_error=False):
         with pbar_man.counter(color='blue', leave=False, total=len(entries), unit='it', desc=desc,
-                              autorefresh=True, position=3) as pbar:
+                              min_interval=Defaults.PBAR_REFRESH_INTERVAL, autorefresh=True, position=3) as pbar:
             for ent in entries:
                 try:
                     n_good, n_bad = self.add_part(dir_path=dir_path, entry=ent, drop_noise=drop_noise,
@@ -334,15 +387,21 @@ class Dataset:
                     self.errors_file.open('a').write(f"{ent.did}\t{msg}\n")
 
     @classmethod
-    def get_paths(cls, dir_path: Path, entry: Entry, compress=False) -> Tuple[Path, Path]:
-        l1_ext = str(entry.did.langs[0])
-        l2_ext = str(entry.did.langs[1])
-        if compress:
-            l1_ext += f'.{DEF_COMPRESS}'
-            l2_ext += f'.{DEF_COMPRESS}'
+    def get_paths(cls, dir_path: Path, entry: Entry, compress=False) -> Union[Path, Tuple[Path, Path]]:
+        """
+        Gets file path for entry in given directory. 
+        Return single path for monolingual entry and tuple of two paths for bilingual entry.
+        Paths may have compress extension such as .gz when compress=True
+        """
+        compress_ext = compress and f'.{DEF_COMPRESS}' or ''
+        l1_ext = str(entry.did.langs[0]) + compress_ext
         l1 = dir_path / f'{entry.did}.{l1_ext}'
-        l2 = dir_path / f'{entry.did}.{l2_ext}'
-        return l1, l2
+        if len(entry.did.langs) == 1: # monolingual
+            return l1
+        else: # biling
+            l2_ext = str(entry.did.langs[1]) + compress_ext
+            l2 = dir_path / f'{entry.did}.{l2_ext}'
+            return l1, l2
 
     def add_part(self, dir_path: Path, entry: Entry, drop_noise=False, compress=False):
         flag_file = dir_path / f'.valid.{entry.did}'
