@@ -6,10 +6,9 @@
 from mtdata.index import Entry, Index, DatasetId
 
 
-def load(index: Index):
+def load_all(index: Index):
     URL_PREFIX = 'https://s3.amazonaws.com/web-language-models/paracrawl'
-    cite = index.ref_db.get_bibtex('espla-etal-2019-paracrawl')
-    cite += '\n' + index.ref_db.get_bibtex('banon-etal-2020-paracrawl')
+    cite = ('espla-etal-2019-paracrawl', 'banon-etal-2020-paracrawl')
     group_id = 'ParaCrawl'
     # === Para crawl corpus
     PARACRAWL_v3 = f'{URL_PREFIX}/release3/%s-%s.bicleaner07.tmx.gz'
@@ -89,3 +88,10 @@ def load(index: Index):
                     in_paths=['en-ja/en-ja.bicleaner05.txt'], in_ext='tsv', cols=cols, cite='',
                     url=f'http://www.kecl.ntt.co.jp/icl/lirg/jparacrawl/release/{version}.0/bitext/en-ja.tar.gz')
         index.add_entry(ent)
+        
+        # JParaCrawl Chinese-Japanese, only version 2 is available
+        if version == '2':
+            ent = Entry(did=DatasetId(group='KECL', name=f'paracrawl', version=version, langs=('zho', 'jpn')),
+                            in_paths=['zh-ja/zh-ja.bicleaner05.txt'], in_ext='tsv', cols=cols, cite='',
+                            url=f'http://www.kecl.ntt.co.jp/icl/lirg/jparacrawl/release/{version}.0/bitext/zh-ja.tar.gz')
+            index.add_entry(ent)
